#include <fireparticle.h>
#include <gameglobal.h>
#include <game.h>

#define FIRE_TOTTIME (FIRE_MAXLIFETIME+FIRE_LIFETIME*(FIRE_SIZES-1)) 
#define FIRE_HALFTIMEFACTOR  (-2.f/FIRE_TOTTIME) 
#define FIREZ_FACTOR (DISTANCE<<3)/FIRE_TOTTIME
#define FIREY_FACTOR (DISTANCE<<5)/FIRE_TOTTIME
#define FIREX_FACTOR -1.f/(FIRE_TOTTIME<<5)
#define FIREANG_FACTOR (M_PI/-128.f)

static int firesortparticle(particle *p1, particle *p2){
    int status=0;
    if(p1->status==P_DEAD) {
        if(p2->status==P_ALIVE) status=-1;
    } else {
        if(p2->status==P_DEAD) status=1;
        else {
            fireparticleinfo *ptr1=(fireparticleinfo *)p1->buff;
            fireparticleinfo *ptr2=(fireparticleinfo *)p2->buff;
            if(ptr2->z>ptr1->z) status=1;
            else if(ptr2->z<ptr1->z) status=-1;
        }
    }
   return status;
}

static int firedrawparticle(particle *plist, int listsize, int index){
    fireparticleinfo *ptr=(fireparticleinfo *)plist[index].buff;
    float scale=(float)DISTANCE/(ptr->z+(DISTANCE<<2));    
    rotate_scaled_sprite(dbuffer,fire[ptr->size],ptr->x+(int)((fire[ptr->size]->w>>1)*scale),ptr->y-(int)((fire[ptr->size]->h>>1)*scale),itofix(ptr->angle),ftofix(scale));
    return P_REMAIN;
}

static int fireupdateparticle(particle *plist, int listsize, int index){
    int w,h;
    int status=P_REMAIN;
    float scale;
    fireparticleinfo *ptr=(fireparticleinfo *)plist[index].buff;
    ptr->x+=ptr->vx;    ptr->y+=ptr->vy;    ptr->z+=ptr->vz;
    ptr->vx+=ptr->ax;    ptr->vy+=ptr->ay;    ptr->vz+=ptr->az; 
    scale=(float)DISTANCE/(ptr->z+(DISTANCE<<2));    
    w=(int)(fire[ptr->size]->w*scale); h=(int)(fire[ptr->size]->h*scale);
    ptr->timer++;
    ptr->angle++;
    if(ptr->vy<0) ;
    else if(ptr->vy==0) {
        ptr->dir=NEWS_PARTDOWN;
    } else if(ptr->vy>=ptr->finy) {
            status=P_EVICT; 
    }
    if(ptr->size==(FIRE_SIZES-1)) {
        if(ptr->timer>FIRE_MAXLIFETIME) {
            ptr->size--;
            ptr->timer=0;
        }
    }  else if(ptr->size>=0) {
        if(ptr->timer>FIRE_LIFETIME) { 
            ptr->timer=0;
            if(ptr->size==0) status=P_EVICT; 
        }
   } 
    if((ptr->x+(w>>1))<=0||(ptr->y+(h>>1))<=0) status=P_EVICT;
    if(ptr->x>=(xres+(w>>1))||ptr->y>=(yres+(h>>1))) status=P_EVICT;
    return status;
}

void fireinit(){
    initparticles(fireparticles, FIRE_PARTICLES);
}

void firedraw(){
    updateparticles(fireparticles, FIRE_PARTICLES, firedrawparticle);
}

void firespawn(){    
    fireparticleinfo newfire;
    
    newfire.ax=0;
    newfire.ay=GRAVITY;
    newfire.az=0; 
    
    newfire.x=ftgunx+ftgun->h-(int)(0.6f*cosf(FIREANG_FACTOR*(128-ftgunang))*(ftgun->w+ftgun->h));
    newfire.y=ftguny-ftgun->h+(int)(sinf(FIREANG_FACTOR*ftgunang)*ftgun->h);
    newfire.z=1;
   
    newfire.vx=(int)((ftgunang-64)*xres*FIREX_FACTOR);
    newfire.vz=FIREZ_FACTOR;
    newfire.vy=(int)(FIREY_FACTOR*tanf(FIREANG_FACTOR*ftgunvang));

    newfire.finy=-newfire.vy;
    newfire.dir=NEWS_PARTUP;
    
    newfire.size=FIRE_SIZES-1;
    
    newfire.timer=0;
    
    newfire.angle=(game_timer%8)<<5;
    
    spawnparticle(fireparticles, FIRE_PARTICLES, &newfire, sizeof(fireparticleinfo));
}

void firesort(){
    sortparticles(fireparticles, FIRE_PARTICLES, firesortparticle);
}

void fireupdate(){
    updateparticles(fireparticles, FIRE_PARTICLES, fireupdateparticle);
}
