//Source file for driver program.

#include <level.h>
#include <log.h>
#include <gamestate.h>
#include <game.h>
#include <gameglobal.h>

//Local.
static int ourgamedone;

//Variant of Binary search
static int startaliveindex(particle *plist, int listsize){
        int low=0;
        int high=listsize-1;
        int median=low+((high-low)>>1);
        while(low<=high) {
            if(plist[median].status==P_ALIVE)
                high=median-1;
            else if(plist[median].status==P_DEAD)
                low=median+1;
            median=low+((high-low)>>1);
        }
        return median;
}

//Collision check for eviction
static int checkcollision(fireparticleinfo *fptr, newsparticleinfo *nptr){
    int ydiff=fptr->y-nptr->y,xdiff=fptr->x-nptr->x;
    if(ydiff<0) ydiff=-ydiff; if(xdiff<0) xdiff=-xdiff; 
    if(ydiff>YDIFF||xdiff>XDIFF) return P_REMAIN;    
    return P_EVICT;
}

//Collision-Detection variant of merge-sort
void performcolld(){
    int startfireindex=startaliveindex(fireparticles, FIRE_PARTICLES);
    int startnewsindex=startaliveindex(newsparticles, NEWS_PARTICLES);
    int fireindex,newsindex;
    fireparticleinfo *fptr;
    newsparticleinfo *nptr;
    
    if(startfireindex<0||startfireindex>=FIRE_PARTICLES) return;
    if(startnewsindex<0||startnewsindex>=NEWS_PARTICLES) return;
    
    fireindex=startfireindex;
    newsindex=startnewsindex;
    fptr=(fireparticleinfo *)fireparticles[fireindex].buff;
    nptr=(newsparticleinfo *)newsparticles[newsindex].buff;    
    while(1){
        if(fireindex>=FIRE_PARTICLES) break;
        if(newsindex>=NEWS_PARTICLES) break;
        if(fireparticles[fireindex].status==P_DEAD||(fptr->z-nptr->z)>ZDIFF) {
            fireindex++;
            fptr=(fireparticleinfo *)fireparticles[fireindex].buff;
        } else if (newsparticles[newsindex].status==P_DEAD||(nptr->z-fptr->z)>ZDIFF) {
            newsindex++;
            nptr=(newsparticleinfo *)newsparticles[newsindex].buff;
        } else {
            if(checkcollision(fptr,nptr)==P_EVICT) {
                evictparticle(fireparticles,FIRE_PARTICLES,fireindex);
                evictparticle(newsparticles,NEWS_PARTICLES,newsindex);
                boomspawn(nptr->x,nptr->y,nptr->z);
                levels[curlevel]->hits++;
                fireindex++;
                fptr=(fireparticleinfo *)fireparticles[fireindex].buff;
                newsindex++;
                nptr=(newsparticleinfo *)newsparticles[newsindex].buff;
            } else {
                if(fptr->z>nptr->z) {
                    newsindex++;
                    nptr=(newsparticleinfo *)newsparticles[newsindex].buff;
                } else if(nptr->z>fptr->z) {
                    fireindex++;
                    fptr=(fireparticleinfo *)fireparticles[fireindex].buff;
                } else {
                    fireindex++;
                    fptr=(fireparticleinfo *)fireparticles[fireindex].buff;
                    newsindex++;
                    nptr=(newsparticleinfo *)newsparticles[newsindex].buff;
                }
            }
        }
    }
}

//Level PreInitialize
void pre_level_init(int levelnum){
    levels[levelnum]->hits=0;
    levels[levelnum]->misses=0;
}

//Draw Level
void level_draw(int levelnum){
        int i=0;
        blit(bg,dbuffer,0,0,0,0,bg->w,bg->h);
        smokedraw();
        masked_blit(factory,dbuffer,0,0,facx,facy,factory->w,factory->h);
        trashdraw();
        firedraw();
        rotate_scaled_sprite(dbuffer,ftgun,ftgunax,ftguny,itofix(-ftgunang),ftofix(1.f+(ftgunvang/128.f)));
        newsdraw();
        boomdraw();
}

//Current Level Draw
void cur_level_draw(){
	level_draw(curlevel);
}

//Current Level Init
void pre_cur_level_init(){
	pre_level_init(curlevel);
}

//First Level
void first_level(){
	curlevel=0;
	pre_cur_level_init();
	log_printf("First Level");
}

//Next Level
void next_level(){
	curlevel++;
	if(curlevel>=nlevels) curlevel=0; 
	pre_cur_level_init();
	log_printf("Next Level, @Level %d",curlevel);
}

//Game Initialization
int game_play_init(){
	int status=-1;	

	//Load Levels
	if(status=load_levels()) return status;
        
        //Particles
        smokeinit();
        fireinit();
        newsinit();
        boominit();
        trashinit();

	first_level();
        ourgamedone=0;
        
	return 0;
}

//Game Quit
void game_play_quit(){
	//Unload Levels
	unload_levels();
}

//Game Input
void game_play_input(){
        int fspw=0;
        if(mouse) {
            int mx=0,my=0;
            get_mouse_mickeys(&mx,&my);
            if(mx) ftgunang=127-((mouse_x<<7)/xres);
            if(my) ftgunvang=32-((mouse_y<<5)/yres);
            if(mouse_b&1) ++fspw;
        }
        if(key[KEY_SPACE]) ++fspw;
        if(key[KEY_LEFT]) ftgunang++;
        if(key[KEY_RIGHT]) ftgunang--;
        if(key[KEY_UP]) ftgunvang++;
        if(key[KEY_DOWN]) ftgunvang--;
        
        if(ftgunang>96) ftgunang=96;
        if(ftgunang<32) ftgunang=32;
        if(ftgunvang>40) ftgunvang=40;
        if(ftgunvang<8) ftgunvang=8;
        ftgunax=ftgunx-(int)(ftgunvang*ADJUST_FACTOR);
        
        if(fspw%2) firespawn();
        
        if(key[KEY_ESC]) ++ourgamedone;
}

//Game Logic
void game_play_logic(){
    if(!(game_timer%100)) { trashupdate(); trashsort(); }
    if(!(game_timer%5)) smokeupdate();
    if(!(game_timer%50)) smokespawn();
    if(!(game_timer%((nlevels-curlevel)<<4))) newsspawn(); 
    if(!(game_timer%(nlevels-curlevel))) { newsupdate(); newssort(); }
    fireupdate(); firesort();  performcolld(); 
    if(!(game_timer%5)) boomupdate();
    boomsort();
    
    //Check Win/Lose
    if(levels[curlevel]->hits>levels[curlevel]->maxhits) ++(levels[curlevel]->win);
    if(levels[curlevel]->misses>levels[curlevel]->maxmisses) ++(levels[curlevel]->lose);
 
    if(levels[curlevel]->lose) state_switch(GAME_LOSE);    
    else if(levels[curlevel]->win) {
        if(curlevel==(nlevels-1)) state_switch(GAME_WIN);
        else next_level();
    } else if(ourgamedone) state_switch(GAME_LOSE); //User Quit. 
    
}

//Game Render
void game_play_render(){
	cur_level_draw();
}
