//Source file for Game States.

#include <header.h>
#include <global.h>
#include <game.h>
#include <gamestate.h>

int state_terminated=0,state_switched=0;
int oldstate=GAME_USUAL,state=GAME_USUAL;

//Logic for States
void state_logic(){

	if(state_terminated) {
		if(state_terminated>0) {
			state_quit();
			terminate_loop();
			state_terminated=-1; //One Time Quit.
		}
		return;
	}

	if(state_switched) --state_switched;

	switch(state){
		case GAME_PLAY: game_play_logic(); break;
                case GAME_WIN: game_win_logic(); break;
                case GAME_LOSE: game_lose_logic(); break;
                case GAME_SPLASH: game_splash_logic(); break;
		default: usual_logic();
	}
}

//Input for States
void state_input(){

	if(state_switched) return;
	if(state_terminated) return;

	switch(state){
		case GAME_PLAY: game_play_input(); break;
                case GAME_WIN: game_win_input(); break;
                case GAME_LOSE: game_lose_input(); break;
                case GAME_SPLASH: game_splash_input(); break;
		default: usual_input();
	}
}

//Render for States
void state_render(){

	if(state_switched) return;
	if(state_terminated) return;

	//Clear Buffer
	clear_bitmap(dbuffer);

	switch(state){
		case GAME_PLAY: game_play_render(); break;
                case GAME_WIN: game_win_render(); break;
                case GAME_LOSE: game_lose_render(); break;
                case GAME_SPLASH: game_splash_render(); break;
		default: usual_render();
	}
	
	//Common for All Games
	blit(dbuffer,screen,0,0,0,0,xres,yres);

}

//Quit for States
void state_quit(){
	switch(state){
		case GAME_PLAY: game_play_quit(); break;
                case GAME_WIN: game_win_quit(); break;
                case GAME_LOSE: game_lose_quit(); break;
                case GAME_SPLASH: game_splash_quit(); break;

		default: usual_quit();
	}
}

//Init for States
int state_init(){
	int status=-1;
	switch(state){
		case GAME_PLAY: status=game_play_init(); break;
                case GAME_WIN: status=game_win_init(); break;
                case GAME_LOSE: status=game_lose_init(); break;
                case GAME_SPLASH: status=game_splash_init(); break;
		default: status=usual_init();
	}
	return status;
}


//Quit for States
void state_switch(int newstate){
	state_quit();

	oldstate=state;
	state=newstate;

	++state_switched;

	if(state_init()) { //Not for GAME_USUAL
		state_quit();
		state=GAME_USUAL;
		(void)state_init();
	}
}

//Quit State Loop
void terminate_state_loop(){
	++state_switched;
	++state_terminated;
}
