//Source file for Level functions.

#include <level.h>
#include <log.h>
#include <util.h>
#include <gameglobal.h>

//Unload Level
void unload_level(level *givlevel){
	if(givlevel){
		//Destroying Level
		free(givlevel);
		#ifdef VERBOSE
			log_printf("Destroyed Level");
		#endif //VERBOSE
	}
}

//Create Level
level *create_level(){
	level *newlevel=NULL;	
	newlevel=(level *)malloc(sizeof(level));	
	if(newlevel) {
            newlevel->maxhits=0;
            newlevel->maxmisses=0;
            newlevel->hits=0;
            newlevel->misses=0;
            newlevel->win=0;
            newlevel->lose=0;
	}
	return newlevel;
}

//Unload all Levels
void unload_levels(){
	int i;
	if(levels) {
		//Unloading Levels
		#ifdef VERBOSE
			log_printf("Destroying Levels");
		#endif //VERBOSE
		for(i=0;i<nlevels;++i){
			//Unload Each level
			unload_level(levels[i]);
			levels[i]=NULL;
		}
		free(levels);
		levels=NULL;
	}
	nlevels=0;
	curlevel=-1;
}


//Load all Levels
int load_levels(){
	int i,maxhits,maxmisses;
	int status=-1;
        FILE *flevels=NULL;
	char filename[MAXSTR];
	
	unload_levels();

	//From: Data/levels.txt
	sprintf(filename,"%s%c%s","Data",PATHSEP,"levels.txt");

	flevels=fopen(filename,"r");
	if(!flevels) return status;

	//Loading Level
	#ifdef VERBOSE
		log_printf("Loading Levels from list: %s",filename);
	#endif //VERBOSE

	nlevels=0; fscanf(flevels,"%d",&nlevels);	
	levels=(level **)malloc(sizeof(level *)*nlevels);
	if(levels) {
		#ifdef VERBOSE
			log_printf("Loading %d Levels",nlevels);
		#endif //VERBOSE
                for(i=0;i<nlevels;++i){
                    levels[i]=NULL;
                }
		for(i=0;i<nlevels;++i){
                        maxhits=0; maxmisses=0;
			fscanf(flevels,"%d %d",&maxhits,&maxmisses);
        		#ifdef VERBOSE
				log_printf("Loading Level %d: %s",i,levelname);
			#endif //VERBOSE
			levels[i]=create_level();                                
                        if(!levels[i]) break;
                        levels[i]->maxhits=maxhits;  levels[i]->maxmisses=maxmisses;
                	curlevel=i;
		}				
	}	
	fclose(flevels);
	return 0;
}
